package article

import (
	"my-blog/test"
	"strconv"
	"testing"
)

var articles = []Article{
	{
		ID:      104,
		Title:   "自己紹介",
		Content: "今更ですが、わたしの自己紹介をさせてください。なんたらかんたら",
	},
	{
		ID:      103,
		Title:   "こんなことがありました",
		Content: "先日、散歩をしていたときに変な出来事に遭遇しました。",
	},
	{
		ID:      102,
		Title:   "仕事について",
		Content: "わたしは新卒のころからずっと続けている仕事があります。",
	},
	{
		ID:      101,
		Title:   "ブログはじめました",
		Content: "このブログではわたしの個人的な事柄について書くつもりです。",
	},
}

func TestGetArticles(t *testing.T) {
	testdb := test.DB(t)
	defer test.Close(t, testdb)
	s := New(testdb)

	got, err := s.GetAll()
	if err != nil {
		t.Fatal("failed to get articles:", err)
	}
	if len(got) != 4 {
		t.Fatal("number of article is not 4:", len(got))
	}
	for i, a := range got {
		want := articles[i]
		test.Eq(t, want.ID, a.ID)
		test.Eq(t, want.Title, a.Title)
		test.Eq(t, want.Content, a.Content)
	}
}

func TestGetArticle(t *testing.T) {
	d := test.DB(t)
	defer test.Close(t, d)

	s := New(d)

	for i := 1; i <= 4; i++ {
		t.Run(strconv.Itoa(i), func(t *testing.T) {
			got, err := s.Get(i + 100)
			if err != nil {
				t.Fatal("failed to get article:", err)
			}

			want := articles[4-i]
			if got == nil {
				t.Fatal("article is nil")
			}
			test.Eq(t, want.ID, got.ID)
			test.Eq(t, want.Title, got.Title)
			test.Eq(t, want.Content, got.Content)
		})
	}

	t.Run("article not found", func(t *testing.T) {
		got, err := s.Get(-1)
		if err == nil {
			t.Fatal("err shouldn't be nil")
		}
		if err.Error() != "failed to scan: sql: no rows in result set" {
			t.Fatalf("unknown error: %v", err)
		}
		if got != nil {
			t.Fatalf("article should be nil: %v", got)
		}
	})
}

func TestCreateArticle(t *testing.T) {
	testdb := test.DB(t)
	defer test.Close(t, testdb)

	s := New(testdb)

	id, err := s.Create("サンプルタイトル", "これはテスト用サンプルテキストです。")
	if err != nil {
		t.Fatal("failed to create article:", err)
	}

	// idのテスト
	test.Eq(t, 1, id)

	// 作成したarticleを取得
	got, err := s.Get(1)
	if err != nil {
		t.Fatal("failed to get article:", err)
	}

	// CreateArticleに渡した内容どおりか確認
	test.Eq(t, "サンプルタイトル", got.Title)
	test.Eq(t, "これはテスト用サンプルテキストです。", got.Content)
}

func TestDeleteArticle(t *testing.T) {
	testdb := test.DB(t)
	defer test.Close(t, testdb)

	s := New(testdb)

	err := s.Delete(101)
	if err != nil {
		t.Fatal("failed to create article:", err)
	}

	// articleを取得
	got, err := s.GetAll()
	if err != nil {
		t.Fatal("failed to get article:", err)
	}

	// 数が減っていればOK
	test.Eq(t, 3, len(got))
}
