package db

import (
	"database/sql"
	"fmt"

	_ "github.com/lib/pq" // go get github.com/lib/pq
)

type Config struct {
	Host     string
	Port     string
	User     string
	Password string
	Database string
	SSL      string
}

func (c *Config) String() string {
	return fmt.Sprintf(
		"host=%s port=%s user=%s password=%s dbname=%s sslmode=disable",
		c.Host, c.Port, c.User, c.Password, c.Database,
	)
}

func New(config *Config) (*sql.DB, error) {
	db, err := sql.Open("postgres", config.String())
	if err != nil {
		return nil, fmt.Errorf("failed to connect database: %w", err)
	}

	if err = db.Ping(); err != nil {
		return nil, fmt.Errorf("failed to run query: %w", err)
	}
	return db, nil
}
