package main

import (
	_ "embed"
	"html/template"
	"log"
	"my-blog/article"
	"my-blog/db"
	"my-blog/handler"
	"net/http"
	"os"
)

func main() {
	dbconfig := &db.Config{
		Host:     getenv("MY_BLOG_DB_HOST", "localhost"),
		Port:     getenv("MY_BLOG_DB_PORT", "5432"),
		User:     getenv("MY_BLOG_DB_USER", "postgres"),
		Password: getenv("MY_BLOG_DB_PASSWORD", "postgres"),
		Database: getenv("MY_BLOG_DB_NAME", "my_blog"),
		SSL:      getenv("MY_BLOG_DB_SSL_MODE", "disable"),
	}
	d, err := db.New(dbconfig)
	if err != nil {
		log.Fatal(err)
	}
	a := article.New(d)
	h := handler.New(
		template.Must(template.ParseFiles("assets/index.html")),
		template.Must(template.ParseFiles("assets/article.html")),
		template.Must(template.ParseFiles("assets/new.html")),
		a,
	)
	http.HandleFunc("/articles/new", h.NewArticle)
	http.HandleFunc("/articles", h.Article)
	http.HandleFunc("/", h.Index)
	log.Fatal(http.ListenAndServe(":8080", nil))
}

func getenv(key, defaultValue string) string {
	if env := os.Getenv(key); env != "" {
		return env
	}
	return defaultValue
}
